//Ethan Alexander Shulman 2020

var camera, player, mouseDown = false,
playing = false,
startTime = 0, generatedX = -5,
objs = [], lastObjCheck = 0, birdUpTex, birdDownTex,
bgAudio, borderLines = new Array(2);

/**Player class, inherits from PhysicsBody2D and Scenery2D
@constructor*/
function Player() {
	PhysicsBody2D.call(this);
	Scenery2D.call(this);
	this.radius = 0.1;
	this.scaleX = this.scaleY = 0.25;
	this.bounce = this.friction = 0;
	this.extrapolate = true;
	this.onCollision = playerCollision;
	this.material =  GetAsset("bird.mat").asset;
	this.queue = 1;
}
Player.prototype = Object.create(PhysicsBody2D.prototype);
Combine(Player.prototype,Scenery2D.prototype);

function playerCollision() {
	//reset after loss
	player.x = 0;
	player.y = 0;
	player.velocityX = 1;
	player.velocityY = 0;
	for (var i = 0; i < objs.length; i++) {
		var o = objs[i];
		o.EnableRendering(false);
		o.EnablePhysics(false);
	}
	objs.length = 0;
	generatedX = -5;
	bgAudio.pause();
	playing = false;
	document.getElementById("mm").style.display = "block";
}


/**Line inherits from PhysicsCollider2D and Scenery2D
@constructor*/
function Line(p1x,p1y,p2x,p2y,size) {
	PhysicsCollider2D.call(this);
	Scenery2D.call(this);
	this.shape = COLLIDER2D_BOX;
	this.material = GetAsset("box.mat").asset;//box material asset
	var dx = p2x-p1x, dy = p2y-p1y,
		len = Math.sqrt(dx*dx+dy*dy);
	this.x = (p1x+p2x)*0.5;//line center point
	this.y = (p1y+p2y)*0.5;
	this.scaleX = len*0.5+size;
	this.scaleY = size;
	this.angle = Math.abs(Math.atan2(dy,dx));//line angle
	this.boundingRadius = len*0.5+size;
}
Line.prototype = Object.create(PhysicsCollider2D.prototype);
Combine(Line.prototype,Scenery2D.prototype);


//Render and update loop
function render() {
	var physTick = true;
	if (playing) {
		var genAmount = Math.floor(player.x)-generatedX;
		if (genAmount > 0) {
			for (var x = generatedX+10, l = generatedX+genAmount+10; x < l; x++) {
				var x1 = Math.random()-0.5+x,
					y1 = (Math.random()-0.5)*3;
				objs.push(NLine(x1,y1,
					x1+Math.random()-0.5,Math.max(-1.4,Math.min(1.4,y1+(Math.random()-0.5)*2)),
					0.02+Math.random()*0.22));
			}
			generatedX += genAmount;
		}
	
		if (tickTime-lastObjCheck > 5000) {
			var deli = 0, delX = player.x-5;
			for (; deli < objs.length; deli++) {
				if (objs[deli].x > delX) break;
			}
			if (deli) {
				for (var i = 0; i < deli; i++) {
					var o = objs[i];
					o.EnableRendering(false);
					o.EnablePhysics(false);
				}
				objs.splice(0,deli);
			}
			lastObjCheck = tickTime;
			Physics2DFreeUnusedMemory();
		}

		physTick = false;
		while (Tick()) {//fixed tick rate using DELTA rate
			physTick = true;
	
			var speed = Math.min(20,1+(tickTime-startTime)/1000/5);
			player.velocityY = Math.max(-5,Math.min(5,player.velocityY+(mouseDown?DELTA*speed:DELTA*-speed*0.5)));
		
			Physics2DUpdate();
		}
	}
	
	//uiPanel.textContent = "Score: "+Math.floor((tickTime-startTime)/100);
	
	//render
	borderLines[0].x = borderLines[1].x = camera.x = player.x+5*display.aspectX-0.2;
	camera.velocityX = player.velocityX;
	player.material.uniforms[3] = player.velocityY>0?birdUpTex:birdDownTex;
	camera.Render(physTick);

	
	requestAnimationFrame(render);
}

//helper function for creating new line
function NLine(p1x,p1y,p2x,p2y,size) {
	var l = new Line(p1x,p1y,p2x,p2y,size);
	l.EnableRendering(true);
	l.EnablePhysics(true);
	return l;
}

function mdown(x,y) {
	if (playing) {
		mouseDown = true;
	} else {
		//main menu, check if clicking start button
		if (y < window.innerHeight*0.5) {
			playing = true;
			document.getElementById("mm").style.display = "none";
			bgAudio.volume = document.getElementById("au").value/100;
			bgAudio.play();
			ResetTickTimer();
			startTime = tickTime;
		}
	}
}

function aspectResize() {
	var ratio = window.innerWidth/window.innerHeight;
	camera.fieldOfView = 5+Math.max(0,ratio-2)*2;
}

document.addEventListener("DOMContentLoaded",function() {
	//initialize engine systems
	SetupSystem();
	if (!SetupGraphics()) {
		alert("Error WebGL not supported!");
		return;
	}
	SetupScene2D();
	
	
	window.addEventListener("mousedown",function(e) {
		mdown(e.clientX,e.clientY);
	});
	window.addEventListener("mouseup",function() {
		mouseDown = false;
	});
		
	
	uiPanel.addEventListener("touchstart",function(e) {
		if (!document.fullscreenElement) document.body.requestFullscreen();
		mdown(e.touches[0].clientX,e.touches[0].clientY);
		//e.preventDefault();
		return false;
	});
	uiPanel.addEventListener("touchend",function(e) {
		if (e.touches.length === 0) mouseDown = false;
		//e.preventDefault();
		return false;
	});
	/*uiPanel.addEventListener("touchcancel",function(e) {
		if (e.touches.length === 0) mouseDown = false;
		e.preventDefault();
		return false;
	});*/
	
	//border lines
	borderLines[0] = NLine(-10,-1.5,10,-1.5,0.08);
	borderLines[1] = NLine(-10,1.5,10,1.5,0.08);
	
	//create player and enable
	player = new Player();
	player.velocityX = 1;
	player.EnableRendering(true);
	player.EnablePhysics(true);
	
	birdUpTex = player.material.uniforms[3];
	birdDownTex = GetAsset("bird_down.png");
	
	bgAudio = GetAsset("music.mp3").asset;
	bgAudio.loop = true;
	
	//load volume from slider
	bgAudio.volume = document.getElementById("au").value/100;
	
	//camera for rendering
	camera = new Camera2D();
	camera.velocityY = 0;
	camera.extrapolate = true;
	camera.fieldOfView = 5;
	camera.clearColor = [0.6,0.2,0.05,1];
	
	//on window resize adjust aspect ratio
	window.addEventListener("resize",aspectResize);
	aspectResize();
	
	DELTA = 1/40;
	render();
});
